;==============================================================================
    include \masm32\include\masm32rt.inc
;==============================================================================

;---------------------------------------------------------
; This macro more or less duplicates the functionality of
; the BASIC language TIMER function.
;---------------------------------------------------------

timer MACRO
    IFNDEF _timer_pc_frequency_
        .data
            align 8
            _timer_pc_frequency_      dq 0
            _timer_pc_count_          dq 0
            _timer_elapsed_seconds_   dq 0
            _timer_initialized_       dd 0
        .code
    ENDIF
    .IF _timer_initialized_ == 0
        invoke QueryPerformanceFrequency, ADDR _timer_pc_frequency_
        inc _timer_initialized_
    .ENDIF
    invoke QueryPerformanceCounter, ADDR _timer_pc_count_
    fild _timer_pc_count_
    fild _timer_pc_frequency_
    fdiv
    fstp _timer_elapsed_seconds_
    EXITM <_timer_elapsed_seconds_>
ENDM

;==============================================================================
    .data
        timesr8 REAL8 50  dup(0.0)
        timesdd dd    400 dup(0)
        min     dd    1
        max     dd    0
    .code
;==============================================================================

Delay proc delay_ms:DWORD

    LOCAL pc_frequency:QWORD
    LOCAL pc_count:QWORD

    invoke QueryPerformanceFrequency, ADDR pc_frequency
    invoke QueryPerformanceCounter, ADDR pc_count

    ;----------------------------------------------------------------------
    ; For a delay in milliseconds the delay count will be (PF * ms) / 1000
    ; This code assumes that the PF will fit in 32 bits.
    ;----------------------------------------------------------------------

    mov eax, DWORD PTR pc_frequency
    mov ecx, delay_ms
    mul ecx
    mov ecx, 1000
    div ecx
    xor edx, edx
    add eax, DWORD PTR pc_count
    adc edx, DWORD PTR pc_count+4
  @@:
    push eax
    push edx
    invoke QueryPerformanceCounter, ADDR pc_count
    pop edx
    pop eax
    cmp DWORD PTR pc_count+4, edx
    jne @B
    cmp DWORD PTR pc_count, eax
    jb  @B
    ret

Delay endp

;==============================================================================
start:
;==============================================================================

    invoke Sleep, 3000

    ;---------------------------------------------------------------
    ; Verify that the timer resolution is in the microsecond range.
    ;---------------------------------------------------------------

    N=0
    REPEAT 50
        fld timer()
        fstp timesr8+N*8
        N=N+1
    ENDM
    N=0
    REPEAT 50
        invoke crt_printf, cfm$("%f\n"), timesr8+N*8
        N=N+1
    ENDM

    invoke Sleep, 1000
    print chr$(13,10,13,10)

    ;----------------------------------------------------
    ; Sample the effective resolution of the tick count.
    ;----------------------------------------------------

    N=0
    REPEAT 50
        fld timer()
        invoke GetTickCount
        push eax
        .WHILE eax == [esp]
            invoke GetTickCount
        .ENDW
        pop eax
        fld timer()
        fsubr
        fstp timesr8+N*8
        N=N+1
    ENDM
    N=0
    REPEAT 50
        invoke crt_printf, cfm$("%f\n"), timesr8+N*8
        N=N+1
    ENDM

    invoke Sleep, 1000
    print chr$(13,10,13,10)

    invoke nseed, 1234567
    N=0
    REPEAT 400
        invoke GetTickCount
        push eax
        invoke nrandom, 10
        mov ebx, eax
        invoke Delay, ebx
        invoke GetTickCount
        pop edx
        sub eax, edx
        sub ebx, eax
        .IF SDWORD PTR ebx < min
            mov min, ebx
        .ENDIF
        .IF SDWORD PTR ebx > max
            mov max, ebx
        .ENDIF
        mov timesdd+N*4, ebx
        N=N+1
    ENDM
    N=0
    REPEAT 50
        print str$(timesdd+N*4),13,10
        N=N+1
    ENDM

    print chr$(13,10)
    print str$(min),9
    print str$(max),13,10
    mov min, 1
    mov max, 0

    invoke Sleep, 1000
    print chr$(13,10,13,10)

    invoke nseed, 1234567
    N=0
    REPEAT 400
        invoke GetTickCount
        mov ebx, eax
        .WHILE eax == ebx
            invoke GetTickCount
        .ENDW
        push eax
        invoke nrandom, 10
        mov ebx, eax
        invoke Delay, ebx
        invoke GetTickCount
        pop edx
        sub eax, edx
        sub ebx, eax
        .IF SDWORD PTR ebx < min
            mov min, ebx
        .ENDIF
        .IF SDWORD PTR ebx > max
            mov max, ebx
        .ENDIF
        mov timesdd+N*4, ebx
        N=N+1
    ENDM
    N=0
    REPEAT 50
        print str$(timesdd+N*4),13,10
        N=N+1
    ENDM

    print chr$(13,10)
    print str$(min),9
    print str$(max),13,10
    print chr$(13,10,13,10)

    inkey "Press any key to exit..."
    exit

;==============================================================================
end start

